package fr.improve.demoserveur.database;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class Database {
	/**
	 * static singleton.
	 */
	private static Database singleton = new Database();
	
	/**
	 * News map.
	 */
	private Map allTheNews = new Hashtable();
	
	/**
	 * Subjects map.	 
	 */
	private Map allTheSubjects = new Hashtable();
	
	public static Database getDataBase() {
		return singleton;									
	}


	/**
	 * Private constructor.	 
	 */
    private Database() {
    	Subject s1 = addSubject("Struts-Layout", "General news concerning Struts-Layout");
		Subject s2 = addSubject("DemoServeurDeNews", "General news concerning the DemoServeurDeNews application");
		addSubject("IBM", "News regarding IBM products : Websphere, WSAD");				
    	
		addNews("Sortable", s1, "Columns are now sortable !", new Date(1031263200000l));
		addNews("Pager", s1, "A pager is available for table.", new Date(1031263200000l));
		addNews("News 1", s2, "Hello World.", new Date(1031176800000l));
		addNews("News 2", s2, "This is a runnable demonstration.", new Date(1031176800000l));
		addNews("News 3", s2, "You can add a news", new Date(1031090400000l));
		addNews("News 4", s2, "You can modify a news", new Date(1031004000000l));
		addNews("News 5", s2, "You can login/logout", new Date(1030917600000l));
    }   

	/**
	 * Check a String for null or empty.
	 */
    private static boolean checkNullOrEmpty(String in_string) {
        boolean l_isNullOrEmpty = false;
        if(in_string == null || in_string.equals(""))
            l_isNullOrEmpty = true;
        return l_isNullOrEmpty;
    }
	
	/**
	 * Add a new news. Used internally.
	 */
    private void addNews(String in_title, Subject in_subject, String in_body, Date in_date) throws InvalidParameterException {
        News l_news = null;
        if(checkNullOrEmpty(in_title))
            throw new InvalidParameterException("Database.addNews : parametre in_title invalide");
        if(in_subject==null)
            throw new InvalidParameterException("Database.addNews : parametre in_title invalide");
        if(checkNullOrEmpty(in_body)) {
            throw new InvalidParameterException("Database.addNews : parametre in_title invalide");
        } else {
            l_news = new News();
            l_news.setTitle(in_title);
            l_news.setSubject(in_subject);
            l_news.setBody(in_body);
            l_news.setDate(in_date);
            int l_nbNews = allTheNews.size();
            String l_id = String.valueOf(l_nbNews);
            l_news.setId(l_id);
            allTheNews.put(l_id, l_news);
            return;
        }
    }
    
    public Subject addSubject(String in_name, String in_description) throws InvalidParameterException {
    	if (checkNullOrEmpty(in_name))
    		throw new InvalidParameterException("Database.addSubject : parametre in_name invalide");
    	if (checkNullOrEmpty(in_description))
    		throw new InvalidParameterException("Database.addSubject : parametre in_description invalide");
    	
    	Subject lc_subject = new Subject();
    	lc_subject.setName(in_name);
    	lc_subject.setDescription(in_description);
    	lc_subject.setId(String.valueOf(allTheSubjects.size()));
    	allTheSubjects.put(lc_subject.getId(), lc_subject);
    	return lc_subject;
    }
    
    public void removeSubject(Subject in_subject) {
    	if (in_subject!=null) {
    		if (getNewsBySubject(in_subject).isEmpty()) {
    			allTheSubjects.remove(in_subject.getId());
    		} else {
    			throw new InvalidParameterException("Database.removeSubject : there are news for this subject");
    		}
    		
    	}
    }

	public void addNews(String in_title, Subject in_subject, String in_body) throws InvalidParameterException {
		addNews(in_title, in_subject, in_body, new Date());
	}	

    public void updateNews(String in_id, String in_title, Subject in_subject, String in_body)
        throws InvalidParameterException {
        News l_news = null;
        if(checkNullOrEmpty(in_id))
            throw new InvalidParameterException("Database.addNews : parametre in_id invalide");
        if(checkNullOrEmpty(in_title))
            throw new InvalidParameterException("Database.addNews : parametre in_title invalide");
        if(in_subject==null)
            throw new InvalidParameterException("Database.addNews : parametre in_subject invalide");
        if(checkNullOrEmpty(in_body))
            throw new InvalidParameterException("Database.addNews : parametre in_body invalide");
    
        l_news = getNewsByPrimaryKey(in_id);
        l_news.setTitle(in_title);
        l_news.setSubject(in_subject);
        l_news.setBody(in_body);               
    }
    
    public void updateSubject(String in_id, String in_name, String in_description) throws InvalidParameterException {
    	if(checkNullOrEmpty(in_id))
            throw new InvalidParameterException("Database.updateSubject : parametre in_id invalide");
        if(checkNullOrEmpty(in_name))
            throw new InvalidParameterException("Database.updateSubject : parametre in_name invalide");
        if(checkNullOrEmpty(in_description))
            throw new InvalidParameterException("Database.updateSubject : parametre in_description invalide");
       // if (getSubjectByName(in_name)!=null) {
       // 	throw new InvalidParameterException("Database.updateSubject : parametre in_name invalide");
        //}
        Subject lc_subject = getSubjectByPrimaryKey(in_id);
        lc_subject.setName(in_name);
        lc_subject.setDescription(in_description);
    }

    public List getAllNews() {
        return new ArrayList(allTheNews.values());
    }
    
    public List getAllSubjects() {    	
    	return new ArrayList(allTheSubjects.values());
    }

    public News getNewsByPrimaryKey(String in_idNews) {        
        News l_result = (News) allTheNews.get(in_idNews);
        return l_result;
    }
    
    public List getNewsBySubject(Subject in_subject) {
    	if (in_subject==null) {
    		return Collections.EMPTY_LIST;
    	}
    	
    	List lc_list = getAllNews();
    	Iterator lc_it = lc_list.iterator();
    	while (lc_it.hasNext()) {
    		News lc_news = (News) lc_it.next();
    		if (!in_subject.equals(lc_news.getSubject())) {
    			lc_it.remove();
    		}    		
    	}
    	return lc_list;
    }
    
    public Subject getSubjectByPrimaryKey(String in_subjectId) {
    	if (in_subjectId==null) {
    		return null;
    	} else {    	
    		return (Subject) allTheSubjects.get(in_subjectId);
    	}
    }
	
	public Subject getSubjectByName(String in_subjectName) {
		if (in_subjectName==null) {
			return null;
		} else {
			Iterator lc_it = allTheSubjects.values().iterator(); 
			while (lc_it.hasNext()) {
				Subject lc_subject = (Subject) lc_it.next();
				if (lc_subject.getName().equalsIgnoreCase(in_subjectName)) {
					return lc_subject;
				}
			}
		}
		return null;
	}
}
